#!/usr/bin/perl
#
# Description:
#   The script reads a gpx waypoint file and parses for
#   usefull information to be stored into an WPT file (waypoint, POI)
#
# History:
#   0.2 - first version
#       released: 04.08.2010
#
#
# $Revision:  $       (c) E. Leschke                       $Date: 2010/08/04$
#-----------------------------------------------------------------------------
my $version = "0.2";
my $DEBUG = 0;
 
use strict;
use warnings;
use English;
use Getopt::Long;
use Data::Dumper;
use File::Basename;
use Pod::Usage;
use LWP::Simple;
use Geo::Gpx;
use Geo::Coordinates::UTM;
use File::Slurp;
use Math::Polygon;


#=====================
# Start configuration
#=====================
my $DEST_DIR = "huetten";

#OS Specific Variables
my $linuxDirDivider = "\/"; my $linuxMoveCommand = "mv"; my $linuxDeleteCommand = "rm"; my $linuxCopyCommand = "cp";
my $windowsDirDivider = "\\"; my $windowsMoveCommand = "move"; my $windowsDeleteCommand = "del"; my $windowsCopyCommand = "copy";
#=====================
# End configuration
#=====================

my ($systemDirDivider, $systemMoveCommand, $systemCopyCommand, $systemDeleteCommand);

if($^O ge "linux" || $^O ge "cygwin" )
{
  $systemDirDivider = $linuxDirDivider;
  $systemMoveCommand = $linuxMoveCommand;
  $systemCopyCommand = $linuxCopyCommand;
  $systemDeleteCommand = $linuxDeleteCommand;
}
elsif($^O ge "MSWin")
{
  $systemDirDivider = $windowsDirDivider;
  $systemMoveCommand = $windowsMoveCommand;
  $systemCopyCommand = $windowsCopyCommand;
  $systemDeleteCommand = $windowsDeleteCommand;
 }
if($^O eq "darwin")
{
  $systemDirDivider = $linuxDirDivider;
  $systemMoveCommand = $linuxMoveCommand;
  $systemDeleteCommand = $linuxDeleteCommand;
}



# forward declaration
sub WriteWpt($); 
 
 
print( "\n" );
print( "----------------------------------------------------------\n" );
print( "|  $0  v$version \n" );
print( "----------------------------------------------------------\n" );

# Check for valid number of arguments
if (($#ARGV < 0))  { pod2usage(2); }

# option handling
our ($help, $man, $debug) = (0,0,0);
our ($GpxFileName, $NorthWest, $SouthEast);

GetOptions(
  'help|h|?'          => \$help,
  'man'               => \$man,
  'debug=i'           => \$debug,
  'gpx=s'             => \$GpxFileName,
  'nw=s'              => \$NorthWest,
  'se=s'              => \$SouthEast,
          );

pod2usage(1) if $help;
pod2usage(-verbose => 2) if $man;
$DEBUG=$debug;

#-------------------------------------------------------------------

          
#-------------------------------------------------------------------
#  open and parse the gpx file, open wpt file
#-------------------------------------------------------------------
print( "Start parsing gpx file $GpxFileName\n" );

# create destination dir if not exist
unless (-d $DEST_DIR)
{
  system("mkdir $DEST_DIR");
}

my ($NorthWest_lat, $NorthWest_lon, $SouthEast_lat, $SouthEast_lon);
my $poly;

if ((defined $NorthWest) && (defined $SouthEast))
{
  $NorthWest =~ m/(.+),(.+)/;
  $NorthWest_lat = $1;
  $NorthWest_lon = $2;

  $SouthEast =~ m/(.+),(.+)/;
  $SouthEast_lat = $1;
  $SouthEast_lon = $2;

  $poly = Math::Polygon->new( [$NorthWest_lat,$NorthWest_lon],
                              [$SouthEast_lat,$NorthWest_lon],
                              [$SouthEast_lat,$SouthEast_lon],
                              [$NorthWest_lat,$SouthEast_lon],
                              [$NorthWest_lat,$NorthWest_lon]
                            );
}

unless (-e $GpxFileName) { die "Can't find GPX file: $GpxFileName"; }

my $GpxContent = read_file($GpxFileName); # entire file in scalar

my $Basename = basename($GpxFileName, '.gpx');
my $WptFile = $DEST_DIR . $systemDirDivider . $Basename . ".wpt";
open( WPT, ">$WptFile" ) or die "Can't open WPT file: $!";

print WPT << "EOH";
B  UTF-8
G  WGS 84
U  1
EOH


# Parse GPX
my $gpx  = Geo::Gpx->new( xml => $GpxContent );
#my $waypoints = $gpx->waypoints();
#if ($DEBUG>0) { print Dumper($waypoints); }

print( "writing wpt file $WptFile\n" );
my $count = 0;
my $inside = 0;
my $bounds = $gpx->bounds();
#  $bounds = {
#    minlat => 57.120939,
#    minlon => -2.9839832,
#    maxlat => 57.781729,
#    maxlon => -1.230902
#  };

# for each of the waypoints
my $iter = $gpx->iterate_waypoints();
my @point;

while ( my $wpt = $iter->() ) 
{
#  print "WPT: ", join( ', ', $wpt->{lat}, $wpt->{lon} ), "\n";
  @point = [$wpt->{lat}, $wpt->{lon}];
#  print Dumper(@point);
#  if ($poly->contains($point))  # is inside the given area?
 
  if ($poly->contains( @point ))
  {
    $inside++;;
    WriteWpt( $wpt );
  }
    $count++;
}

close WPT;

print "\ndone ($count entries processed, $inside entries inside and written).\n";




####################################################################
#  s u b r o u t i n e s
####################################################################

#          {
#            'link' => {
#                      'text' => "Eintrag in Jo's H\x{fc}ttenliste",
#                      'href' => 'http://www.tourenwelt.info/huettenliste/huette.php?huette=16'
#                    },
#            'ele' => '3300',
#            'lat' => '35.3542',
#            'name' => 'Akaiwa hachigokan',
#            'sym' => 'Residence',
#            'lon' => '138.745',
#            'cmt' => 'Homepage: http://www.dumbo33.net/shop/asobu/akaiwa.html'
#          },

# ---------------------------------------
# WriteWpt 
# ---------------------------------------
sub WriteWpt($)
{
  my ($wpt) = @_;
  my ($name, $latitude, $longitude, $elevation, $comment, $link, $link_txt, $link_href, $sym);
  my ($id, $InfoFile);
  
  #print Dumper($wpt);
  
  $name = $$wpt{name};
  $latitude = $$wpt{lat};
  $longitude = $$wpt{lon};
  $elevation = $$wpt{ele};
  if (!defined $elevation) { $elevation = 0; }
  $comment = $$wpt{cmt};
  $sym = $$wpt{sym};
  $link = $$wpt{link};
  $link_txt = $$link{text};
  $link_href = $$link{href};
  
  $link_href =~ m/http(.*)huette=(\d+)/;
  $id = $2;
  
  if ($DEBUG>0) { print "id=$id: lat=${latitude}, lon=${longitude}, ele=${elevation}\n"; }
  
  # write the additional info text file
  $InfoFile = $DEST_DIR . $systemDirDivider . "${id}.txt";
  open( INFO, ">$InfoFile" ) or die "Can't open WPT INFO file: $!";
  print INFO "Hütte: $name\015\012";
  print INFO "lat=${latitude}, lon=${longitude}, ele=${elevation}\015\012";
  print INFO "$comment\015\012";
  print INFO "$link_href\015\012";
  close INFO;

  # convert into right format of wpt
  if ($latitude >= 0)
  { $latitude .= 'ºN'; }
  else
  { $latitude = abs($latitude); $latitude .= 'ºS'; }
  if ($longitude >= 0)
  { $longitude .= 'ºE'; }
  else
  { $longitude = abs($longitude); $longitude .= 'ºW'; }

  # write the waypoint
  print WPT "W  $name A ${latitude} ${longitude} 27-MAR-62 00:00:00 ${elevation} ${comment}\n";
  print WPT "w ${sym},0,0.0,16316664,248,1,55,${link_href},100.0,,-1,0\n";
  print WPT "a \\Storage Card\\TwoNavData\\Data\\WPT\\${id}.txt,0\n";
    
}



#==============================
# End of script
#==============================

__END__



#^L
#==============================
# POD follows
#==============================

=head1 NAME

B<bb2gpx>

The script converts GPX file into WPT file including linked info in text file.

=head1 SYNOPSIS

B<bb2gpx [options]>

  Options:
    --help | -h | -?            brief help message
    --man                       full documentation
    --gpx=s                     file name of the input gpx file to be converted (mandatory)
    --nw=s                      north west point of the relevant area (--nw=lat,lon)
    --se=s                      south east point of the relevant area (--se=lat,lon)


=head1 OPTIONS

=over 8

=item --help | -h | -?

Print a brief help message and exits.

=item --man

Prints the manual page and exits.

=item --gpx

Specifies the GPX file as input. The resulting WPT file will have the same basename.


=item --nw=s   --se=s

Specifies the area of interest as follow:

--nw	north west point of the relevant area (--nw=lat,lon)

--se	south east point of the relevant area (--se=lat,lon)

=back

=head1 DESCRIPTION

   The script reads the given GPX file with waypoint inside and parses for
   usefull information to be stored into an WPT file (waypoint, POI).

=head1 PREREQUISITES

This tool is entirly written in perl and requires the following Perl modules installed:

  - English
  - Data::Dumper
  - Getopt::Long
  - Pod::Usage
  - File::Basename
  - LWP::Simple
  - HTML::FormatText
  - Geo::Gpx
  - Geo::Coordinates::UTM
  - File::Slurp
  - Math::Polygon


=head1 AUTHORS

This version of the tool is completely written by Ekkart Leschke.

=head1 COPYRIGHT

Copyright (c) 2010 Ekkart Leschke. All rights reserved.
This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.
See the Perl Artistic.txt for details.

=cut


#--end--
